#include "stdio.h"
#include "time.h"
#include "defsc573.h"
#include "cdefsc573.h"

#define BUFF_SIZE 4096
#define INDEX_OFFSET 1024
#define DMC0_START_ADDRESS 0x80000000
#define TCB_SIZE 4

#define ENABLE_EMDMA_NORMAL
#define ENABLE_MDMA_MSIZE4_NORMAL
#define ENABLE_MDMA_MSIZE32_NORMAL

#define ENABLE_EMDMA_CIRCULAR
#define ENABLE_MDMA_MSIZE4_CIRCULAR
#define ENABLE_MDMA_MSIZE32_CIRCULAR


#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG |= 0x1;\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG |= 0x1;

#define DISABLE_MDMA(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = 0;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = 0;

//Macro to wait for DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));\
		*pREG_DMA##DMA_CHANNEL##_STAT=ENUM_DMA_STAT_IRQDONE;

#define EMDMA_CONFIG(EMDMA_CHANNEL,EMDMA_CTL,INT_ADDRSTART, INT_MOD, INT_COUNT, EXT_ADDRSTART, EXT_MOD, EXT_COUNT)\
	*pREG_EMDMA##EMDMA_CHANNEL##_INDX0=INT_ADDRSTART;\
	*pREG_EMDMA##EMDMA_CHANNEL##_MOD0=INT_MOD;\
	*pREG_EMDMA##EMDMA_CHANNEL##_CNT0=INT_COUNT;\
	*pREG_EMDMA##EMDMA_CHANNEL##_INDX1=EXT_ADDRSTART;\
	*pREG_EMDMA##EMDMA_CHANNEL##_MOD1=EXT_MOD;\
	*pREG_EMDMA##EMDMA_CHANNEL##_CTL=EMDMA_CTL

#define EMDMA_CIRCULAR_CONFIG(EMDMA_CHANNEL,EMDMA_CTL,INT_ADDRSTART, INT_MOD, INT_COUNT, EXT_ADDRSTART, EXT_MOD, EXT_COUNT,EXT_ADDRBASE, EXT_LENGTH)\
	*pREG_EMDMA##EMDMA_CHANNEL##_INDX0=INT_ADDRSTART;\
	*pREG_EMDMA##EMDMA_CHANNEL##_MOD0=INT_MOD;\
	*pREG_EMDMA##EMDMA_CHANNEL##_CNT0=INT_COUNT;\
	*pREG_EMDMA##EMDMA_CHANNEL##_INDX1=EXT_ADDRSTART;\
	*pREG_EMDMA##EMDMA_CHANNEL##_MOD1=EXT_MOD;\
	*pREG_EMDMA##EMDMA_CHANNEL##_BASE=EXT_ADDRBASE;\
	*pREG_EMDMA##EMDMA_CHANNEL##_BUFLEN=EXT_LENGTH;\
	*pREG_EMDMA##EMDMA_CHANNEL##_CTL=EMDMA_CTL

#define WAIT_FOR_EMDMA_OVER(EMDMA_CHANNEL)\
	while((*pREG_EMDMA##EMDMA_CHANNEL##_CTL&BITM_EMDMA_CTL_DMAS0)==0);\
	while((*pREG_EMDMA##EMDMA_CHANNEL##_CTL&BITM_EMDMA_CTL_DMAS0)!=0);\
	while((*pREG_EMDMA##EMDMA_CHANNEL##_CTL&BITM_EMDMA_CTL_DMAS1)!=0);

#define DISABLE_EMDMA(EMDMA_CHANNEL)\
	*pREG_EMDMA##EMDMA_CHANNEL##_CTL=0;

#define SRC_DMA_CFG1	    (ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08)
#define SRC_DMA_CFG2 	   (ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_PSIZE08)
#define DST_DMA_CFG1 		(BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_XCNT_INT)
#define DST_DMA_CFG2 		(BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_XCNT_INT)

#define SRC_DMA_CFG3	 	(ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define SRC_DMA_CFG4		(ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define SRC_DMA_CFG5	    (ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08)
#define SRC_DMA_CFG6	    (ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_PSIZE08)

#define DST_DMA_CFG3 	 	(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define DST_DMA_CFG4 	 	(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define DST_DMA_CFG5 		(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_XCNT_INT)
#define DST_DMA_CFG6 		(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_XCNT_INT)


#define SRC_DMA_MSIZE1 4
#define DST_DMA_MSIZE1 4

#define SRC_DMA_MSIZE2 32
#define DST_DMA_MSIZE2 32


#define EMDMA_CTL_VALUE_WRITE_CIRCULAR 		BITM_EMDMA_CTL_EN|BITM_EMDMA_CTL_TRAN|BITM_EMDMA_CTL_CBEN
#define EMDMA_CTL_VALUE_READ_CIRCULAR   	BITM_EMDMA_CTL_EN|BITM_EMDMA_CTL_CBEN
#define EMDMA_CTL_VALUE_WRITE 				BITM_EMDMA_CTL_EN|BITM_EMDMA_CTL_TRAN
#define EMDMA_CTL_VALUE_READ   				BITM_EMDMA_CTL_EN


#define START_CYCLE_COUNT\
		ccount=0;\
		clock_start = clock();

#define TAKE_CYCLE_SNAPSHOT\
	clock_stop = clock();\
	cycles_array[ccount] = clock_stop - clock_start;\
	ccount++;\
	clock_start = clock_stop;

#define PRINT_CYCLES_AND_TIME\
	int cc;\
	for(cc=0;cc<ccount;cc++)\
	{							 	  \
		abs_time[cc]=cycles_array[cc]/450;		  \
		printf("\nCycle count for snapshot %d =%d, Time = %fus \n",cc,cycles_array[cc],abs_time[cc]);\
	}


void Mem_Copy_MDMA_MSIZE4_Circ_Write(unsigned int srcadd, unsigned int dstadd, unsigned int count, unsigned int dstaddbase, unsigned int buflen);
void Mem_Copy_MDMA_MSIZE4_Circ_Read(unsigned int srcadd, unsigned int dstadd, unsigned int count, unsigned int srcaddbase, unsigned int buflen);
void Mem_Copy_MDMA_MSIZE32_Circ_Write(unsigned int srcadd, unsigned int dstadd, unsigned int count, unsigned int dstaddbase, unsigned int buflen);
void Mem_Copy_MDMA_MSIZE32_Circ_Read(unsigned int srcadd, unsigned int dstadd, unsigned int count, unsigned int srcaddbase, unsigned int buflen);

